#ifndef EEPROM_25XX640
#define EEPROM_25XX640

; Interface to MAXIM 25XX640 SPI eeproms
;
; the CS pin is assumed to be MAX25XX640_CS

#include "../include/spi.asm"

#define EEPROM_INST_READ 0x03	; Read data from memory array beginning at selected address
#define EEPROM_INST_WRITE 0x02	; Write data to memory array beginning at selected address
#define EEPROM_INST_WREN 0x06	; Set the write enable latch (enable write operations)
#define EEPROM_INST_WRDI 0x04	; Reset the write enable latch (disable write operations)
#define EEPROM_INST_RDSR 0x05	; Read status register
#define EEPROM_INST_WRSR 0x01	; Write status register


	SEGU data
eeprom_25xx640_param1 DS 1
eeprom_25xx640_param2 DS 1
eeprom_25xx640_param3 DS 1
	
	SEG code

	
;-------------
; EEPROM_DISABLE
;
; Disable writing to the serial eeprom device

EEPROM_DISABLE
	bcf		MAX25XX640_CS
	movlw	EEPROM_INST_WRDI
	call	SPI_READWRITE8
	bsf		MAX25XX640_CS
	return
	

;--------------
; EEPROM_START_WRITE
;
; Starts a write operation to the EEPROM... takes the address in
; eeprom_25xx640_param1 (ms byte) and param2 (ls byte)

EEPROM_START_WRITE
	bcf		MAX25XX640_CS
	movlw	EEPROM_INST_WREN
	call	SPI_READWRITE8
	bsf		MAX25XX640_CS

	bcf		MAX25XX640_CS
	movlw	EEPROM_INST_WRITE
	call	SPI_READWRITE8
	
	movf	eeprom_25xx640_param1,W
	movwf	spi_high
	movf	eeprom_25xx640_param2,W
	movwf	spi_low
	
	call	SPI_READWRITE16
	
	return
	
;----------------
; EEPROM_WRITE_BYTE
;
; If called after a START_WRITE appends the passed byte to the list of bytes
; to be written, results undefined otherwise...

EEPROM_WRITE_BYTE
	call	SPI_READWRITE8
	return

;----------------
; EEPROM_END_WRITE
;
; Ends a write operation

EEPROM_END_WRITE
	bsf		MAX25XX640_CS
	return
	

;----------------
; EEPROM_SIMPLE_WRITE
;
; Writes eeprom_25xx640_param3 to the address in the first two param regs
;
; Note:  I simply copied the START_WRITE code so as to avoid overflowing
;        stack.  Otherwise the calling depth can be 3 just for this one routine.

EEPROM_SIMPLE_WRITE
	bcf		MAX25XX640_CS
	movlw	EEPROM_INST_WREN
	call	SPI_READWRITE8
	bsf		MAX25XX640_CS
	bcf		MAX25XX640_CS
	movlw	EEPROM_INST_WRITE
	call	SPI_READWRITE8
	
	movf	eeprom_25xx640_param1,W
	movwf	spi_high
	movf	eeprom_25xx640_param2,W
	movwf	spi_low
	
	call	SPI_READWRITE16

	movf	eeprom_25xx640_param3,W

	call	SPI_READWRITE8

	bsf		MAX25XX640_CS
	return

;-------------------
; EEPROM_SIMPLE_READ
;
; Reads byte into eeprom_25xx640_param3 from the address in the first two
; param regs.

EEPROM_SIMPLE_READ
	bcf		MAX25XX640_CS
	movlw	EEPROM_INST_READ
	call	SPI_READWRITE8
	
	movf	eeprom_25xx640_param1,W
	movwf	spi_high
	movf	eeprom_25xx640_param2,W
	movwf	spi_low
	
	call	SPI_READWRITE16
	
	movlw	0
	call	SPI_READWRITE8
	
	movf	spi_temp2,W
	movwf	eeprom_25xx640_param3

	bsf		MAX25XX640_CS
	return
	
;----------------------
; EEPROM_WAIT
;
; Waits for the EEPROM to finish writing to the memory array

EEPROM_WAIT
	bcf		MAX25XX640_CS
	movlw	EEPROM_INST_RDSR
	call	SPI_READWRITE8
	
	movlw	0
	call	SPI_READWRITE8
	
	bsf		MAX25XX640_CS
	btfsc	spi_temp2,0
	goto	EEPROM_WAIT
	
	return
		
#endif
