#ifndef SPI_ASM
#define SPI_ASM

; This file defines routines for communicating over a SPI bus.
;
; SPI_CLK, SPI_DIN, and SPI_DOUT must be predefined

	SEGU data
	
spi_low DS 1
spi_high DS 1
spi_temp DS 1
spi_temp2 DS 1
spi_cnt1 DS 1
spi_cnt2 DS 1
	
	SEG code	

;------------
; SPI_READ16
;
; Reads a 16 bit number from the SPI bus
;
; 4+8*8+4+8*8+2=138

SPI_READ16   
    movlw   8
    movwf   spi_cnt1
    clrf    spi_high
    bcf STATUS,C
spi_read_loop  
    bsf SPI_CLK
    btfsc   SPI_DOUT
    bsf STATUS,C  
    rlf spi_high,F

    bcf SPI_CLK

    decfsz  spi_cnt1,F 
    goto    spi_read_loop

    movlw   8
    movwf   spi_cnt2
    clrf    spi_low
    bcf STATUS,C
spi_read_loop2        
    bsf SPI_CLK       
    btfsc   SPI_DOUT  
    bsf STATUS,C
    rlf spi_low,F
    bcf SPI_CLK       
    decfsz  spi_cnt2,F
    goto    spi_read_loop2
    
    return

;------------
; SPI_READ12
;
; Reads a 12 bit number from the SPI bus
;
; 4+8*4+4+8*8+2=106

SPI_READ12   
    movlw   4
    movwf   spi_cnt1
    clrf    spi_high
    bcf STATUS,C
spi_read12_loop  
    bsf SPI_CLK
    bcf SPI_CLK
    btfsc   SPI_DOUT
    bsf STATUS,C  
    rlf spi_high,F
    decfsz  spi_cnt1,F 
    goto    spi_read12_loop

    movlw   8
    movwf   spi_cnt2
    clrf    spi_low
    bcf STATUS,C
spi_read12_loop2        
    bsf SPI_CLK       
    bcf SPI_CLK       
    btfsc   SPI_DOUT  
    bsf STATUS,C
    rlf spi_low,F
    decfsz  spi_cnt2,F
    goto    spi_read12_loop2
    
    return

	
;-------------
; SPI_READWRITE16
;
; spi_high and spi_low have the data to be sent
; and they are filled with the received data
;
; 12 + SPI_READWRITE8*2= 12+109*2=230

SPI_READWRITE16
	movf	spi_high,W
	call	SPI_READWRITE8
	movf	spi_temp2,W
	movwf	spi_high
	
	movf	spi_low,W
	call	SPI_READWRITE8
	movf	spi_temp2,W
	movwf	spi_low
	retlw 	0
	
;-------------
; SPI_READWRITE8
;
; W has the data to be written
; spi_temp2 has the received data
;
; they will be overwitten with the received data
;
; 3+13*8+2=109


SPI_READWRITE8
	movwf	spi_temp

	movlw	8
	movwf	spi_cnt1
spi_rwloop1
	rlf	spi_temp,F
	btfsc	STATUS,C
	bsf		SPI_DIN
	btfss	STATUS,C
	bcf		SPI_DIN	
	
	bsf		SPI_CLK
	
	bcf		STATUS,C
	btfsc	SPI_DOUT
	bsf		STATUS,C
	rlf		spi_temp2,F
	
	bcf		SPI_CLK
	decfsz	spi_cnt1,F
	goto	spi_rwloop1
	
	retlw 0	

#endif
