#ifndef UART_3110
#define UART_3110

#include "../include/spi.asm"

; Interface to MAXIM 3110 UART using my SPI interface
;
; the CS pin is assumed to be MAX3110_CS

#include "../include/uart_3110.h"

; The following are the configuration bits that can be set



#define UART_3110_CONFIG WRITE_CONFIGURATION | BAUD_RATE_3110

	SEGU data
	
uart_3110_temp DS 1	
uart_3110_temp2 DS 1
uart_rx_buf DS 1
	
	SEG code

;----------------
; UART_INIT
;
; Called to initialize the UART with 
; a default configuration
;

UART_232_INIT

	movlw	HIGH(UART_3110_CONFIG)
	movwf	uart_3110_temp
	movlw	LOW(UART_3110_CONFIG)
	movwf	uart_3110_temp2
	call	UART_232_INIT_PARM
	
	return
	

;----------------
; UART_INIT_PARM
;
; Called to initialize the UART with a passed config register
; in uart_3110_temp and uart_3110_temp2
;
; if it does not succeed it returns nonzero in W

UART_232_INIT_PARM
	movf	uart_3110_temp,W
	movwf	spi_high
	movf	uart_3110_temp2,W
	movwf	spi_low
	
	bcf		MAX3110_CS
	call	SPI_READWRITE16
	bsf		MAX3110_CS
	
	movlw	HIGH(READ_CONFIGURATION)
	movwf	spi_high
	clrf	spi_low
	bcf		MAX3110_CS
	call	SPI_READWRITE16
	bsf		MAX3110_CS
	
	movf	spi_low,W
	subwf	uart_3110_temp2,W
	btfss	STATUS,Z
	retlw 1
	
	movlw	B'00111111'
	andwf	spi_high,F
	movf	uart_3110_temp,W
	andlw	B'00111111'
	subwf	spi_high,W
	btfss	STATUS,Z
	retlw 1	; the initialization failed!
	
	retlw 0


;----------------
; UART_RX
;
; receives a byte into spi_low
; if no bytes are available 0 is returned in W otherwise
; 1 is returned...
;
; 6(7)+SPI_READ16=6(7)+138=144(145)
;

UART_232_RX
	bcf		SPI_DIN
	bcf		MAX3110_CS
	call	SPI_READ16
	bsf		MAX3110_CS
	btfss	spi_high,7
	retlw	1
	btfss	spi_high,2
	retlw	2
	retlw	0
	
;----------------
; UART_TX
;
; the byte to send is in W
;

UART_232_TX
	; first have to wait to make sure the uart has room
	; for our character....
	movwf	uart_3110_temp
	
uart_232_tx_wait
	movlw	HIGH(READ_CONFIGURATION)
	movwf	spi_high
	clrf	spi_low
	bcf		MAX3110_CS
	call	SPI_READWRITE16
	bsf		MAX3110_CS
	
	btfss	spi_high,6
	goto	uart_232_tx_wait
	
	movf	uart_3110_temp,W
	movwf	spi_low
	movlw	HIGH(WRITE_DATA)
	movwf	spi_high
	bcf		MAX3110_CS
	call	SPI_READWRITE16
	bsf		MAX3110_CS
	retlw 0

;----------------
; UART_232_PARITY_CHECK
;
; calculates the parity of the byte in spi_low and compares it against
; the received parity bit in spi_high.  If the conditions for odd parity
; are met, W=0 on return. otherwise w=1
;

UART_232_PARITY_CHECK
	movf	spi_low,W
	movwf	uart_3110_temp
	swapf	uart_3110_temp,F
	xorwf	uart_3110_temp,W
	movwf	uart_3110_temp
	rrf		uart_3110_temp,F
	rrf		uart_3110_temp,F
	xorwf	uart_3110_temp,W
	movwf	uart_3110_temp
	rrf		uart_3110_temp,F
	xorwf	uart_3110_temp,F
	movlw	1
	andwf	uart_3110_temp,F
	andwf	spi_high,W
	subwf	uart_3110_temp,W
	btfss	STATUS,Z
	retlw	0
	retlw	1

#endif
